unit Unit2;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  ComObj, ActiveX, AxCtrls, Classes, AutoServ_TLB, StdVcl;

type
  TTest = class(TAutoObject, IConnectionPointContainer, ITest)
  private
    { Private declarations }
    FConnectionPoints: TConnectionPoints;
    FConnectionPoint: TConnectionPoint;
    FEvents: ITestEvents;
    { note: FEvents maintains a *single* event sink. For access to more
      than one event sink, use FConnectionPoint.SinkList, and iterate
      through the list of sinks. }
  public
    procedure Initialize; override;
    procedure AfterConstruction; override;
    destructor Destroy; override;
    property Events:  ITestEvents read FEvents;
  protected
    { Protected declarations }
    property ConnectionPoints: TConnectionPoints read FConnectionPoints
      implements IConnectionPointContainer;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function Get_Text: WideString; safecall;
    function Get_Visible: WordBool; safecall;
    function Get_Width: Integer; safecall;
    procedure AddLine(const Line: WideString); safecall;
    procedure NewFile; safecall;
    procedure OpenFile(const FileName: WideString); safecall;
    procedure SaveFile(const FileName: WideString); safecall;
    procedure Set_Text(const Value: WideString); safecall;
    procedure Set_Visible(Value: WordBool); safecall;
    procedure Set_Width(Value: Integer); safecall;
    function Get_LineCount: Integer; safecall;
    function Get_Lines(Index: Integer): WideString; safecall;
    procedure Set_Lines(Index: Integer; const Value: WideString); safecall;
  end;

var
ClientList: TThreadList = nil;

implementation

uses ComServ, Unit1;

procedure TTest.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as ITestEvents;
end;

procedure TTest.Initialize;
begin
  inherited Initialize;
  FConnectionPoints := TConnectionPoints.Create(Self);
  if AutoFactory.EventTypeInfo <> nil then
    FConnectionPoint := FConnectionPoints.CreateConnectionPoint(
      AutoFactory.EventIID, ckSingle, EventConnect)
  else FConnectionPoint := nil;
end;


function TTest.Get_Text: WideString;
begin
  try
   Form1.LockForm;
   Result := Form1.Memo1.Text;
  finally
   Form1.UnlockForm;
  end;
end;

function TTest.Get_Visible: WordBool;
begin
 try
  Form1.LockForm;
  Result := Form1.Visible;
 finally
  Form1.UnlockForm;
 end;
end;

function TTest.Get_Width: Integer;
begin
  try
    Form1.LockForm;
    Result := Form1.Width;
  finally
    Form1.UnlockForm;
  end;
end;

procedure TTest.AddLine(const Line: WideString);
begin
  try
    Form1.LockForm;
    Form1.Memo1.Lines.Add(Line);
  finally
    Form1.UnlockForm;
  end;
end;

procedure TTest.NewFile;
begin
  try
    Form1.LockForm;
    Form1.SpeedButton1Click(nil);
  finally
    Form1.UnlockForm;
  end;
end;

procedure TTest.OpenFile(const FileName: WideString);
begin
  try
    Form1.LockForm;
    if Length(FileName)>0 then
       Form1.Memo1.Lines.LoadFromFile(FileName)
    else
       Form1.SpeedButton2Click(nil);
  finally
    Form1.UnlockForm;
  end;
end;

procedure TTest.SaveFile(const FileName: WideString);
begin
  try
    Form1.LockForm;
    if Length(FileName) > 0 then
        Form1.Memo1.Lines.SaveToFile(FileName)
    else
        Form1.SpeedButton3Click(nil);
  finally
    Form1.UnlockForm;
  end;
end;

procedure TTest.Set_Text(const Value: WideString);
begin
  try
   Form1.LockForm;
   Form1.Memo1.Text := Value;
  finally
   Form1.UnlockForm;
  end;
end;

procedure TTest.Set_Visible(Value: WordBool);
begin
 try
   Form1.LockForm;
   Form1.Visible := Value;
  finally
   Form1.UnlockForm;
  end;
end;

procedure TTest.Set_Width(Value: Integer);
begin
 try
   Form1.LockForm;
   Form1.Width := Value;
 finally
   Form1.UnlockForm;
end;

end;

function TTest.Get_LineCount: Integer;
begin
  try
    Form1.LockForm;
    Result := Form1.Memo1.Lines.Count;
  finally
    Form1.UnlockForm;
  end;
end;

function TTest.Get_Lines(Index: Integer): WideString;
begin
  try
     Form1.LockForm;
     Result := Form1.Memo1.Lines[Index];
  finally
     Form1.UnlockForm;
  end;
end;

procedure TTest.Set_Lines(Index: Integer; const Value: WideString);
begin
  try
    Form1.LockForm;
    Form1.Memo1.Lines[Index] := Value;
  finally
    Form1.UnlockForm;
  end;
end;

procedure TTest.AfterConstruction;
var
  L:TList;
begin
  inherited;
  try
    L := ClientList.LockList;
    L.Add(Self);
  finally
    ClientList.UnlockList;
  end;
end;

destructor TTest.Destroy;
var
  L:TList;
  N:integer;
begin
  try
    L := ClientList.LockList;
    N := L.IndexOf(Self);
    if N >= 0 then L.Delete(N);
  finally
    ClientList.UnlockList;
  end;
  inherited;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TTest, Class_Test,
    ciMultiInstance, tmFree);
  ClientList := TThreadList.Create;
finalization
  ClientList.Free;
end.
